package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.FieldAccess;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Removes the bean definition to prevent overriding messages 
 * and duplicate exceptions.
 */
public class DefaultListableBeanFactoryCBP extends JavassistClassBytecodeProcessor {
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.util");
    cp.importPackage("org.zeroturnaround.javarebel.integration.spring");

    CtMethod m = ctClass.getDeclaredMethod("registerBeanDefinition");   
    m.insertBefore("if (SpringBeanReloader.isReloadingBeanDefinitions()) {\n" +
        "  beanDefinitionMap.remove($1);" +
        "  beanDefinitionNames.remove($1);" +
    "}");

    try {
      m = ctClass.getDeclaredMethod("getBeanDefinition");
      m.insertBefore("SpringBeanReloader.checkAndReloadBeanDefinitions();");
    }
    catch (NotFoundException e ) { /*OK*/}
    
    try {
      m = ctClass.getDeclaredMethod("containsBeanDefinition");
      m.insertBefore("SpringBeanReloader.checkAndReloadBeanDefinitions();");
    }
    catch (NotFoundException e ) { /*OK*/}
    
    try {
      m = ctClass.getDeclaredMethod("getBeanDefinitionNames");
      m.insertBefore("SpringBeanReloader.checkAndReloadBeanDefinitions();");
    }
    catch (NotFoundException e ) { /*OK*/}
    
    try {
      m = ctClass.getDeclaredMethod("getBeanDefinitionCount");
      m.insertBefore("SpringBeanReloader.checkAndReloadBeanDefinitions();");
    }
    catch (NotFoundException e ) { /*OK*/}
    
    try {
      m = ctClass.getDeclaredMethod("isBeanNameInUse");
      m.insertBefore("SpringBeanReloader.checkAndReloadBeanDefinitions();");
    }
    catch (NotFoundException e ) { /*OK*/}
    
    
    /*
     * Two changes:
     * 1) Makes a copy of beanDefinitionNames to avoid concurrent modification.
     * 2) Only instanciates fresh singletons
     */
    m = ctClass.getDeclaredMethod("preInstantiateSingletons"); 
    m.instrument(new ExprEditor() {
      public void edit(FieldAccess f) throws CannotCompileException {
        if (f.getFieldName().equals("beanDefinitionNames")) {
          f.replace("{" +
          		"$_ = new java.util.ArrayList($proceed());\n" +
          		"for (java.util.Iterator i = $_.iterator(); i.hasNext();) {" +
          		"  if (getSingleton((String) i.next()) != null) i.remove();\n" +
          		"}" +
          "}");
        }
      }
    });
  }
}
