package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Avoids error "Duplicate &lt;global-method-security&gt; detected" during a
 * reload of Spring Security bean definitions.
 * <p>
 * Requires Spring 2.1 M3 or later.
 */
public class GlobalMethodSecurityBeanDefinitionParserCBP extends
        JavassistClassBytecodeProcessor {
    public void process(ClassPool cp, ClassLoader cl, CtClass ctClass)
            throws Exception {
    	
    	/* 
    	 * Method BeanDefinitionRegistry.removeBeanDefinition(String) was added since Spring version 2.1 M3 (2007-07-31)
    	 * 
    	 * Checking if this method exists:
    	 */
    	CtClass beanDefinitionRegistryClass = cp.get("org.springframework.beans.factory.support.BeanDefinitionRegistry");
    	try {
    		beanDefinitionRegistryClass.getDeclaredMethod("removeBeanDefinition");
    	} catch (NotFoundException e) {
    		return; 
    	}
    	
        CtMethod m =
                ctClass.getDeclaredMethod(
                        "registerDelegatingMethodDefinitionSource");
        m.insertBefore(
                "if ($1.getRegistry().containsBeanDefinition(org.springframework.security.config.BeanIds.DELEGATING_METHOD_DEFINITION_SOURCE)) { $1.getRegistry().removeBeanDefinition(org.springframework.security.config.BeanIds.DELEGATING_METHOD_DEFINITION_SOURCE); }");                                         
    }
}
