package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

public class LocalVariableTableParameterNameDiscovererCBP extends JavassistClassBytecodeProcessor {

	public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
		cp.importPackage("org.zeroturnaround.javarebel");
		cp.importPackage("java.util");
		cp.importPackage("java.lang.reflect");

		try {
			ctClass.getDeclaredField("parameterNamesCache");
			ctClass.getDeclaredField("classReaderCache");
		} catch (NotFoundException e) {
			return;
		}

        ctClass.addInterface(cp.get(ClassEventListener.class.getName()));

		CtConstructor[] constructors = ctClass.getConstructors();
		for (int i = 0; i < constructors.length; i++) {
			CtConstructor constructor = constructors[i];
			constructor.insertAfter("ReloaderFactory.getInstance().addClassReloadListener($0);");
		}

	    ctClass.addMethod(CtNewMethod.make(
	            "public int priority() {" +
	            "	return ClassEventListener.PRIORITY_DEFAULT;" +
	            "}",
	    ctClass));

	    CtMethod onClassEventMethod = CtNewMethod.make(
	            "public void onClassEvent(int eventType, Class clazz) {" +
		    	"	if (classReaderCache.remove(clazz) != null) {" +
		    	"		List keys = new ArrayList(parameterNamesCache.keySet());" +
		    	"		for (Iterator i = keys.iterator(); i.hasNext(); ) {" +
		    	"			Member m = (Member) i.next();" +
		    	"			if (clazz.equals(m.getDeclaringClass())) {" +
		    	"				parameterNamesCache.remove(m);" +
		    	"			}" +
		    	"		}" +
		    	"	}" +
	            "}",
	    ctClass);
	    ctClass.addMethod(onClassEventMethod);
	}

}
