package org.zeroturnaround.javarebel.integration.spring.cbp;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Processes <code>org.springframework.core.io.support.PathMatchingResourcePatternResolver</code.
 * 
 * @author Rein Raudjärv
 */
public class PathMatchingResourcePatternResolverCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.springframework.core.io");
    cp.importPackage("org.zeroturnaround.javarebel.integration.spring");

    try {
      CtMethod origMethod = ctClass.getDeclaredMethod("findPathMatchingResources");
      CtMethod copyMethod = CtNewMethod.copy(origMethod, "_findPathMatchingResources", ctClass, null);
      ctClass.addMethod(copyMethod);

      origMethod.setBody("{" +
          "  String rootDirPath = determineRootDir($1);" +
          "  Resource[] rootDirResources = getResources(rootDirPath);" +
          "  Resource[] result = _findPathMatchingResources($1);" +
          "  return PathMatchingResourcePatternResolverUtil.filter(this, rootDirPath, rootDirResources, result);" +
          "}");
    } catch (NotFoundException e) {
    }
  }

}
